#include "ToneMapper.h"
#include "FrameBufferObject.h"


namespace CHCDemoEngine 
{


ToneMapper::ToneMapper()
{
}


float ToneMapper::CalcImageKey(ColorBufferObject *colorBuffer) const
{
	float totalLum = 0;

	// we assume that we have a floating point rgba texture
	float *pixels = (float *)colorBuffer->ReadTexture();

	const int w = colorBuffer->GetHeight();
	const int h = colorBuffer->GetWidth();

	// Get full scene luminance
	for (int i = 0; i < h; ++ i)
	{
		for (int j = 0; j < w; ++ j)
		{
			const int idx = i * w + j;

			float r = pixels[4 * idx + 0];
			float g = pixels[4 * idx + 1];
			float b = pixels[4 * idx + 2];

			float pixLum = 0.2125f * r + 0.7154f * g + 0.0721f * b;

			totalLum += log(pixLum + 1e-3f);
		}
	}

	const float key = exp(totalLum) / (float)(w * h);

	static float previousKey = key;

	//float delta = imageKey - mPreviousImageKey;
	// Constaint change for temporal coherence
	//const float maxChange = 5-e2f;
	//Clamp(delta, -maxChange, maxChange);

	const float factor = 0.5f;

	const float imageKey = key * factor + previousKey * (1.0f - factor);
	previousKey = imageKey;
	
	delete [] pixels;

	return imageKey;
}

}